import pickle
import requests
from bs4 import BeautifulSoup

import uva.localdb as localdb

BASE_URL = 'https://onlinejudge.org/index.php'

LOGIN_PARAMS = {
    'option': 'com_comprofiler',
    'task': 'login',
}

SUBMIT_PARAMS = {
    'option': 'com_onlinejudge',
    'Itemid': 8,
    'page': 'save_submission'
}

UHUNT_BASE_API_URL = 'https://uhunt.onlinejudge.org/api'
UHUNT_UNAME2UID_API_URL = UHUNT_BASE_API_URL + '/uname2uid'
UHUNT_SUBS_USER_LATEST_API_URL = UHUNT_BASE_API_URL + '/subs-user-last'

NOT_AUTHORIEZED_ERROR_STRING = 'You are not authorised to view this resource'
SUBMISSION_SUCESS_MESSAGE = 'mosmsg=Submission+received+with+ID+'


def login(username, password):
    session = requests.Session()
    r = session.get(BASE_URL)
    soup = BeautifulSoup(r.content, 'html5lib')
    form = soup.find('form', id='mod_loginform')
    inputs = form.find_all('input', type='hidden')

    form_data = {
        'username': username,
        'passwd': password,
        'remember': 'yes'
    }
    for tag in inputs:
        form_data[tag['name']] = tag['value']

    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
    }

    # TODO add error checks on this call
    response = session.post(BASE_URL, params=LOGIN_PARAMS, headers=headers, data=form_data)

    res = response.content.decode("utf-8")

    if NOT_AUTHORIEZED_ERROR_STRING in res:
        print('You are not authorize')
    elif 'My Account' in res and 'Logout' in res:
        print('You are logged in')
    else:
        print('There was an error')

    localdb.save_cookies(pickle.dumps(session.cookies))

    # TODO add error checking on this call
    p = requests.get(UHUNT_UNAME2UID_API_URL + '/' + username)

    localdb.save_login_data(username, p.content.decode("utf-8"))


def get_latest_subs(count):
    uhunt_uid = localdb.read_uhunt_uid()
    url = f'{UHUNT_SUBS_USER_LATEST_API_URL}/{uhunt_uid}/{count}'
    # TODO add error checks for this call
    submissions = requests.get(url)
    return submissions.content.decode("utf-8")


def logout():
    localdb.purge()


def submit(problem_id, filepath, language):
    cookies = localdb.read_cookies()
    session = requests.session()
    session.cookies.update(pickle.loads(cookies))

    files = {
        'localid': (None, problem_id),
        'language': (None, language),
        'codeupl': (filepath, open(filepath, 'rb')),
    }

    response = session.post(BASE_URL, params=SUBMIT_PARAMS, files=files)
    res = response.content.decode("utf-8")

    if NOT_AUTHORIEZED_ERROR_STRING in res:
        print('You are not authorize')
    elif SUBMISSION_SUCESS_MESSAGE in res:
        index = res.find(SUBMISSION_SUCESS_MESSAGE)
        end = res.find('"', index)
        submission_id = res[index + len(SUBMISSION_SUCESS_MESSAGE):end]
        print(submission_id)
        #TODO call this endpoint to fetch the submission status and see if everything is good
        #https://uhunt.onlinejudge.org/api/subs-user/88772/28133692

    else:
        print('There was an error')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['uva', 'uva.commands', 'uva.localdb']

package_data = \
{'': ['*']}

install_requires = \
['bs4>=0.0.1,<0.0.2',
 'html5lib>=1.1,<2.0',
 'pickledb>=0.9.2,<0.10.0',
 'requests>=2.28.1,<3.0.0',
 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['uva = uva.main:app']}

setup_kwargs = {
    'name': 'uva',
    'version': '0.1.0',
    'description': '',
    'long_description': '# Uva Command Line Tool\n\nThis is an Uva Command line tool that can help with Uva online judge problem submissions.',
    'author': 'Aleksandar Markovic',
    'author_email': 'aleksamarkoni@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import typer
from rich import print

import uva.commands as commands

app = typer.Typer()

# @app.command()
# def login(username: str = typer.Option(..., prompt=True),
#           password: str = typer.Option(..., prompt=True, confirmation_prompt=True, hide_input=True)):
#     uva = Uva()
#     uva.login(username, password)


@app.command()
def login(username: str = 'aleksamarkoni', password: str = 'nikokaoona'):
    commands.login(username, password)


@app.command()
def logout():
    commands.logout()


@app.command()
def latest_subs(count: int = 10):
    subs = commands.get_latest_subs(count)
    print(subs)


# 1 for ANSI, 2 for JAVA, 3 for C++, 4 for Pascal, 5 for C++11, 6 for Python.
@app.command()
def submit(problem_id: int, filepath: str, language: int):
    commands.submit(problem_id, filepath, language)


if __name__ == "__main__":
    app()
